from dotenv import load_dotenv
from fastapi import FastAPI
from fastapi.openapi.utils import get_openapi
from starlette.middleware.authentication import AuthenticationMiddleware

load_dotenv()

from app.routers import (
    chargepoint_v1,
    id_token_v1,
    meter_value_v1,
    ocpp_v1,
    transaction_v1,
    user_v1
)
from app.util.websocket_auth_backend import BasicAuthBackend

def create_ocpp_app():
    app_ocpp = FastAPI(
        responses={404: {"description": "Not found"}},
    )
    app_ocpp.include_router(ocpp_v1.router)
    app_ocpp.add_middleware(AuthenticationMiddleware, backend=BasicAuthBackend())
    
    return app_ocpp

def custom_openapi():
    if app.openapi_schema:
        return app.openapi_schema
    openapi_schema = get_openapi(
        title="simple-ocpp-cs",
        version="0.1.0",
        summary="Simple implementation of a basic OCPP 2.0.1 compliant central system (backend) for EV charging stations",
        routes=app.routes,
    )
    app.openapi_schema = openapi_schema
    return app.openapi_schema

def create_app():
    app = FastAPI(
        responses={404: {"description": "Not found"}},
    )
    
    app.include_router(chargepoint_v1.router, prefix="/v1")
    app.include_router(id_token_v1.router, prefix="/v1")
    app.include_router(user_v1.router, prefix="/v1")
    app.include_router(meter_value_v1.router, prefix="/v1")
    app.include_router(transaction_v1.router, prefix="/v1")
    app.mount(path="/v1/ocpp", app=create_ocpp_app())
    app.openapi = custom_openapi

    return app

app = create_app()
